#!/usr/bin/python
# This program is free software. It comes without any warranty, to the extent
# permitted by applicable law. You may use it, redistribute it and/or modify
# it, in whole or in part, provided that you do so at your own risk and do not
# hold the developers or copyright holders liable for any claim, damages, or
# other liabilities arising in connection with the software.
# 
# Developed by Gerald Knizia, 2010--2012.

# create and execute an asymptote script for a jmol export section.
# needs asymptote (vector graphics script language) and epstopdf.
#
# usage: see end of file (or just run it)
#                                   - Gerald Knizia, 21.9.2012

AtomScale = 0.8
BondScale = AtomScale
MakeAtomLabels = True
MakeAtomLabelsAlsoForH = [False,MakeAtomLabels][0]
MakeAtomLabelsAlsoForC = True
BondThreshold = 1.2  # normally 1.3, but here we want to avoid a C--Cu bond.
BondInnerRadius = .9
BondThinning = .9
BaseLineWidth = 1.
#BaseLineWidth = 0.5
BondBrightness = 0.

Halos = True
#Halos = False
HaloBrightness = [.25,-.05]

MirrorAxis = "" # axis to mirror in screen space


GeometryScale = 1.    # Factor by which to multiply input coordinates. By default: input in angstrom.


ToAng2006 = 0.529177249   # 2006 value.
ToAng =     0.5291772108  # molpro default.
ToAngCp2k = 0.529177211   # CP2K default


from sys import argv
from numpy import *
import re
from scipy.linalg import expm
from copy import copy
from os import system, environ, path

def ReadFile(FileName):
   File = open(FileName, "r")
   Text = File.read()
   File.close()
   return Text
def WriteFile(FileName, Text):
   File = open(FileName, "w")
   File.write(Text)
   File.close()

Marked = []

ElementNames = ("H He Li Be B C N O F Ne Na Mg Al Si P S Cl Ar K Ca Sc Ti V Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr Rb Sr Y Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te I Xe Cs Ba La Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu Hf Ta W Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn").split()

ElementNames.append('x') # dummy site (printed in shade)
ElementNames.append('t') # hubbard t site
ElementNames.append('U') # hubbard U site

ElementNames2 = dict([(e.lower(),e) for e in ElementNames ])

def ElementNumberFromName(s):
   return ElementNames.index(s) + 1

def ReadAtomicRadii(iType):
   # number 	symbol 	name 	empirical  	Calculated 	van der Waals 	covalent 	triple-bond covalent radii metallic radii
   import os.path
   import inspect  # <- hack to get path of current script file.
   ScriptFilePath = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
   #Input = ReadFile(environ["HOME"]+"/dev/wiki-atom-radii.txt")
   Input = ReadFile(path.join(ScriptFilePath,"wiki-atom-radii.txt"))
   Lines = Input.splitlines()
   Data = {}
   for Line in Lines:
      ls = Line.split('\t')
      assert(int(ls[0]) == len(Data)+1)
      try:
         Data[int(ls[0])] = float(ls[iType])
      except:
         Data[int(ls[0])] = None
   return Data
#AtomicRadii = ReadAtomicRadii(4)
#CovalentRadii = ReadAtomicRadii(6)
#print "CovalentRadii = %s" % repr(CovalentRadii)
CovalentRadii = {1: 38.0, 2: 32.0, 3: 134.0, 4: 90.0, 5: 82.0, 6: 77.0, 7: 75.0, 8: 73.0, 9: 71.0, 10: 69.0, 11: 154.0, 12: 130.0, 13: 118.0, 14: 111.0, 15: 106.0, 16: 102.0, 17: 99.0, 18: 97.0, 19: 196.0, 20: 174.0, 21: 144.0, 22: 136.0, 23: 125.0, 24: 127.0, 25: 139.0, 26: 125.0, 27: 126.0, 28: 121.0, 29: 138.0, 30: 131.0, 31: 126.0, 32: 122.0, 33: 119.0, 34: 116.0, 35: 114.0, 36: 110.0, 37: 211.0, 38: 192.0, 39: 162.0, 40: 148.0, 41: 137.0, 42: 145.0, 43: 156.0, 44: 126.0, 45: 135.0, 46: 131.0, 47: 153.0, 48: 148.0, 49: 144.0, 50: 141.0, 51: 138.0, 52: 135.0, 53: 133.0, 54: 130.0, 55: 225.0, 56: 198.0, 57: 169.0, 58: None, 59: None, 60: None, 61: None, 62: None, 63: None, 64: None, 65: None, 66: None, 67: None, 68: None, 69: None, 70: None, 71: 160.0, 72: 150.0, 73: 138.0, 74: 146.0, 75: 159.0, 76: 128.0, 77: 137.0, 78: 128.0, 79: 144.0, 80: 149.0, 81: 148.0, 82: 147.0, 83: 146.0, 84: None, 85: None, 86: 145.0, 87: None, 88: None, 89: None, 90: None, 91: None, 92: None, 93: None, 94: None, 95: None, 96: None, 97: None, 98: None, 99: None, 100: None, 101: None, 102: None, 103: None, 104: None, 105: None, 106: None, 107: None, 108: None, 109: None, 110: None, 111: None, 112: None, 113: None, 114: None, 115: None, 116: None}
# ^- embedded now to remove dependency on finding .txt path.

for HubbardElem in ['U', 't', 'x']:
   r = 77.
   CovalentRadii[ElementNumberFromName(HubbardElem)] = r

#for Element in [1,6,7,8,29]:
   #print "Radius: %i = %s (atomic)  %s (cov.)" % (Element, AtomicRadii[Element], CovalentRadii[Element])

def HC(h_): # hex color
   h = h_
   if type(h_) is int:
      h = '%06x' % h_
   if h.startswith('0x'):
      h = h[2:]
   assert(type(h) is str and len(h)==6)
   c = lambda s: int(s,16)/255.
   return (c(h[0:2]), c(h[2:4]), c(h[4:6]))

def ModBrightness(c, f):
   """modifies the brightness of a color by mixing with white or black.
   f: -1 .. +1.   -1 -> black, +1 -> white."""
   if type(c) is str:
      c = HC(c)
   if ( f == 0. ):
      return c
   if f > 0:
      d = (1.,1.,1.)
   else:
      d = (0.,0.,0.)
      f *= -1
   assert(0 <= f <= 1.)
   def lerp(a,b,f):
      return (1.-f)*a + f*b
   return (lerp(c[0],d[0],f),lerp(c[1],d[1],f),lerp(c[2],d[2],f))


# see ~/dev/qt-gl/MakeColorCodes.py
AtomColors = {}
ElementColors = map(HC,[0xffffff,0xffc0cb,0xb22121,0xff1493,0x00ff00,0xd3d3d3,0x87cee6,0xff0000,0xdaa520,0xff1493,0x0000ff,0x228b22,0x696969,0xdaa520,0xffaa00,0xffff00,0x00ff00,0xff1493,0xff1493,0x696969,0xff1493,0x696969,0xff1493,0x696969,0x696969,0xffaa00,0xff1493,0x802828,0x802828,0x802828,0xff1493,0xff1493,0xff1493,0xff1493,0x802828,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0x696969,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xa020f0,0xff1493,0xff1493,0xffaa00,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xdaa520,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493,0xff1493])

# these things overwrite the default colors.
AtomColors = {
   "H": (1.,1.,1.),
   "C": (.6,.6,.6),
   "O": (1.,.1,.1),
   "N": (.3,.6,1.),
   #"N": (.5,.5,1.),
   "Br": HC("802828"),
   "Fe": (0.5, 0.8, .6),
   "Cu": (1.0, .6, .3),
   "Cr": (1.0, .6, .3),
   "x": (0.95,0.95,0.95),
   #"t": (0.95,0.95,0.75),
   "t": (0.9,0.9,0.9),
   "U": (0.3,0.3,0.3),
}

class FAtom:
   def __init__(self, iElem, xyz, Id):
      self.iElem = iElem
      self.xyz = xyz
      self.Id = Id
   def CovRadius(self):
      return CovalentRadii[self.iElem]/100.
   def AtomRadius(self):
      #return AtomicRadii[self.iElem]/100.
      return CovalentRadii[self.iElem]/100. * AtomScale
   def Name(self):
      return ElementNames[self.iElem-1]
   def Color(self):
      if self.Name() in AtomColors:
         c = AtomColors[self.Name()]
      else:
         c = ElementColors[self.iElem-1]
      if len(Marked) != 0:
         def lerp(a,b,f):
            return (1-f)*a + f*b;
         c = array(c)
         if ( self.Id in Marked ):
            c = lerp(c, array([1.,0.0,0.0]), .9)
         else:
            g = .8
            c = lerp(c, array([g,g,g]), .6)
         c = tuple(c)
      return c
   def __str__(self):
      return "%3s  %10.4f %10.4f %10.4f" % (self.ElementNames[self.iElem-1], self.xyz[0], self.xyz[1], self.xyz[2])

def dist(a,b):
   d = a - b
   return dot(d,d)**.5
def length(a):
   return dot(a,a)**.5
def cross(a,b):
   return array([a[1]*b[2]-a[2]*b[1], a[2]*b[0]-a[0]*b[2], a[0]*b[1]-a[1]*b[0]])
def normalize(a):
   return a/length(a)

def ReadXyzFile(FileName,CoordScale=1.):
   Text = ReadFile(FileName)
   Lines = Text.splitlines()
   Atoms = []
   for Line in Lines[2:]:
      if ( Line.strip() == "" ):
         continue
      Elem, x,y,z = Line.split()
      Elem = ElementNames2[Elem.lower()]
      Atoms.append(FAtom(ElementNumberFromName(Elem),
                         CoordScale*array([float(x), float(y), float(z)]),
                         len(Atoms)+1))
   return Atoms

class FMolecule:
   def __init__(self, Atoms):
      self.Atoms = Atoms
      Bonds = []
      # find bonds.
      for i in range(len(Atoms)):
         for j in range(i+1,len(Atoms)):
            r1 = Atoms[i].CovRadius()
            r2 = Atoms[j].CovRadius()
            r = dist(Atoms[i].xyz, Atoms[j].xyz)
            if ( r < BondThreshold * (r1+r2) ):
               Bonds.append((i,j))

      self.Bonds = Bonds

   def __str__(self):
      out = ""
      iat = 0
      for at in self.Atoms:
         bds = []
         for bd in self.Bonds:
            if ( iat in bd ):
               bds.append(bd)
         if ( len(bds) == 0 ):
            s = ""
         else:
            s = "   %s" % bds
         out += "  %-2s  %12.5f %12.5f %12.5f%s" % (at.Name(), at.xyz[0],at.xyz[1],at.xyz[2], s) + "\n"
         iat += 1
      return out


def MakeRenderScript(m, Eye, Lookat, Up_):
   Dir = normalize(Lookat-Eye)
   Up = normalize(Up_)
   Right = cross(Dir, Up)
   Up = normalize(cross(Right,Dir))
   xmat = array([Right,Up,Dir])
   #xmat = array([Up,Right,Dir])
   if ('x' in MirrorAxis): xmat[0,:] *= -1
   if ('y' in MirrorAxis): xmat[1,:] *= -1
   if ('z' in MirrorAxis): xmat[2,:] *= -1
   #print xmat
   def pos2d(v):
      return dot(xmat,v - Lookat)

   xyz = [ FAtom(at.iElem, pos2d(at.xyz), at.Id) for at in m.Atoms ]

   AtomScale = 0.6

   AsyLines = []
   AsyLines.append("import settings;")
   #AsyLines.append("papertype = \"nullpaper\";")
   #AsyLines.append("paperwidth = 200*mm;")
   #AsyLines.append("paperheight = 200*mm;")
   #AsyLines.append("papertype = \"\";")
   AsyLines.append("from plain_pens access *;")
   AsyLines.append("unitsize(20*mm);")
   AsyLines.append("real atom_scale = 1.0;")
   AsyLines.append("pen  atom_pen = linewidth(1.1*%f) + rgb(0,0,0);" % BaseLineWidth)
   AsyLines.append("pen  dummy_pen = linewidth(1.0*%f) + linetype(new real[] {4,3}) + rgb(0.7,0.7,0.7);" % BaseLineWidth)
   AsyLines.append("pen  bond_pen = linewidth(0.9*%f) + rgb(0,0,0);" % BaseLineWidth)
   #AsyLines.append("void fld(path o, pen paint, pen stroke) { fill(o, paint); draw(o, stroke); }")
   AsyLines.append("void fld(path o, pen paint, pen stroke) { filldraw(o, paint, stroke); }")
   AsyLines.append('defaultpen(font("OT1","cmss","m","n"));')
   AsyLines.append('pair FontShift = - (0,0.04);')

   #xyz.sort(lambda a,b: -cmp(a.xyz[2], b.xyz[2]))
   Lines = []

   # make atoms.
   for at in xyz:
      pos = at.xyz
      col = at.Color()
      col0 = col
      if ( Halos ):
         if max(col) >= .5:
            col0 = ModBrightness(col,+HaloBrightness[1])
            col1 = ModBrightness(col,+HaloBrightness[0])
         else:
            col0 = ModBrightness(col,-HaloBrightness[0])
            col1 = ModBrightness(col,-HaloBrightness[1])
      pen = 'atom_pen'
      if ( at.Name() == "x" ): pen = 'dummy_pen'
      Lines.append((pos[2], "fld(circle((%.5f,%.5f), %.5f), rgb(%.2f,%.2f,%.2f), %s);" % (\
         pos[0], pos[1], AtomScale * at.AtomRadius(),
         col0[0], col0[1], col0[2], pen )))
      if ( Halos ):
         dr = AtomScale * at.AtomRadius()
         Lines.append((pos[2], "fill(circle((%.5f,%.5f), %.5f), rgb(%.2f,%.2f,%.2f));" % (\
            #pos[0]-.05*dr, pos[1]+.03*dr, .7*dr,
            pos[0]-.07*dr, pos[1]+.04*dr, .7*dr,
            #pos[0]-.07*dr, pos[1]+.03*dr, .6*dr,
            col1[0], col1[1], col1[2] )))
      # make atom Labels
      if ( MakeAtomLabels ):
         if ( at.Name() == "H" and not MakeAtomLabelsAlsoForH ): continue
         if ( at.Name() == "C" and not MakeAtomLabelsAlsoForC ): continue
         pos = at.xyz
         col = at.Color()
         fcol = (0.,0.,0.)
         if ( at.Name() in ["t", 'x'] ): continue
         if ( at.Name() == "U" ): fcol = (1.,1.,1.)
         Lines.append((pos[2], "label(scale(%s)*baseline('%s'),FontShift+(%.5f,%.5f),rgb(%.2f,%.2f,%.2f));" % (\
            [1.8,1.0][at.Name() == "H"],
            at.Name(), pos[0], pos[1], fcol[0], fcol[1], fcol[2])))

   def MakeBondLine(at0, at1):
      ar = AtomScale * at0.AtomRadius()
      br = AtomScale * at1.AtomRadius()
      a0 = at0.xyz
      b0 = at1.xyz
      #if ( abs(a0[2] - b0[2]) > 0.3 or True ): # hm..
      if ( abs(a0[2] - b0[2]) > 0.3 or False ): # hm..
         ar *= BondInnerRadius
         br *= BondInnerRadius
      ab = normalize(b0 - a0)

      # move away somewhat, depending on the bond radius
      a = a0 + ar * ab
      b = b0 - br * ab

      # center.
      #m = (a0+b0)*.5
      #m = 0.25*((a0+b0)*.5 + 3*(a+b)*.5)
      m = (a+b)*.5

      BondWidth = 0.14 * BondScale

      # perpendicular direction to bond in 2d space
      perp = normalize(array([-ab[1], ab[0], 0.0])) * BondWidth

      mf = BondThinning
      sh1 = [ a + perp, m + perp*mf, m - perp*mf, a - perp ]
      sh2 = [ b + perp, m + perp*mf, m - perp*mf, b - perp ]
      def drw1(sh, col):
         path = ""
         for pos in sh:
            path += "(%.5f,%.5f)--" % (pos[0],pos[1])
         path += "cycle"
         return "fld(%s, rgb(%.2f,%.2f,%.2f), bond_pen );" % (path, col[0],col[1],col[2])
      Line = drw1(sh1, ModBrightness(at0.Color(),BondBrightness)) + "\n"\
           + drw1(sh2, ModBrightness(at1.Color(),BondBrightness))
      return Line

   # make bonds.
   for Bond in m.Bonds:
      at0 = xyz[Bond[0]]
      at1 = xyz[Bond[1]]
      Line = MakeBondLine(at0,at1)
      Lines.append((0.5*(at0.xyz[2] + at1.xyz[2]), Line))

   # sort from back to front (approximately. Lines[0] is an average z value)
   Lines.sort(lambda a,b: -cmp(a[0], b[0]))

   return "\n".join(AsyLines) + "\n\n" + "\n".join([ Line[1] for Line in Lines])

def CleanEpsPaperSize(FileName):
   # asymptote inserts bogus page size definitions (invokes dvips with -t option),
   # and it cannot be stopped. Filter out the page size stuff.
   #%%BeginPaperSize: Letter
   #/setpagedevice where
   #{ pop << /PageSize [612 792] >> setpagedevice }
   #{ /letter where { pop letter } if }
   #ifelse
   #%%EndPaperSize
   with open(FileName, "r") as File:
      Text = File.read()
   ib = Text.find("\n%%BeginPaperSize:")
   s = "\n%%EndPaperSize"
   ie = Text.find(s)
   if ib != -1:
      Text = Text[:ib] + Text[ie+len(s):]
      with open(FileName, "w") as File:
         File.write(Text)

def Render(FileName, m, Eye, Lookat, Up_):
   Script = MakeRenderScript(m, Eye, Lookat, Up_)
   WriteFile(FileName, Script)
   FileNameEps = FileName.replace(".asy",".eps")
   system("asy %s" % FileName)
   CleanEpsPaperSize(FileNameEps)
   #system("asy -f pdf %s" % FileName)
   system("epstopdf %s" % FileNameEps)
   #system("ps2pdf -dEPSCrop %s %s" % (FileName.replace(".asy",".eps"), FileName.replace(".asy",".pdf")))


def mkp(s):
   return array(map(float,s.split()))

def mk(s): return map(int,s.split(','))


def FindClosingParenthesis(s,iPos,Open="(",Close=")"):  # multi-char version.
   # returns position of ')' matching '(' at s[iPos]
   assert(s.startswith(Open))
   lo = len(Open)
   lc = len(Close)
   iLoopLevel = 0
   iPos = 0
   while iPos + 1 < len(s):
      if s[iPos:].startswith(Open):
         iLoopLevel += 1
         iPos += lo
      elif s[iPos:].startswith(Close):
         iLoopLevel -= 1
         iPos += lc
      else:
         iPos += 1
      if iLoopLevel == 0:
         break
   if iLoopLevel != 0:
      raise Exception("imbalanced parenthesis in '%s'" % s)
   return iPos


def RemoveJscriptComments(Line):
   Line = Line.strip()
   #if "//" in Line:
      #Line = Line[:Line.index('//')].rstrip()
   if Line.startswith("#"):
      return ""
   Parts = Line.split('/*')
   L = [Parts[0]]
   for Part in Parts[1:]:
      if "*/" not in Part:
         raise Exception("unbalanced /* .. */ comment in '%s'" % Line)
      L.append(Part[Part.index('*/')+2:])
   return "".join(L)
         

#re.match("wheeee\({(.*)}\)",'wheeee({12 13})').groups()

# note: re must be enclosed in ( ), otherwise it is matched, but not returned.
def RenderJmolState(FileName, OutputFile = None):
   global Marked
   JmolFile = ReadFile(FileName)
   if OutputFile is None:
      OutputFile = path.splitext(FileName)[0] + '.asy'
   print "*reading '%s' -> write '%s'" % (FileName, OutputFile)
   Lines = [o.strip() for o in JmolFile.splitlines()]
   FileNameXyz = None
   for Line_ in Lines:
      Line = RemoveJscriptComments(Line_)
      # see "on_jmol_navigation.txt".
      # select ({12 13 15 16});
      m = re.match("select \({(.*)}\);",Line)
      if m:
         #print "Matched select: %s" % m.group(1)
         Marked = []
         for s in m.group(1).split():
            if ':' not in s:
               Marked.append(int(s))
            else:
               ls = s.split(':')
               Marked += list(range(int(ls[0]),int(ls[1])+1))
         for i in range(len(Marked)):
            Marked[i] += 1
         print "   selected atoms: %s [%i total]" % (" ".join(map(str,Marked)), len(Marked))
         continue
      #  moveto 0.0 { -688 -711 -142 11.61} 100.0 0.0 0.06 {-0.17972994 1.7473251 -0.34660503} 7.1966133 {0.0 0.0 0.0} 0.4249439 15.2643795 0.0;;
      m = re.match("moveto .* {(.*)} (.*) {(.*)} (.*) {(.*)} (.*)\;", Line)
      # moveto /* time, axisAngle */ 1.0 { -688 -711 -142 11.61} /* zoom, translation */  100.0 0.0 0.06  /* center, rotationRadius */ {-0.17972994 1.7473251 -0.34660503} 7.1966133 /* navigation center, translation, depth */ {0.0 0.0 0.0} 0.43010154 15.449701 0.0;
      if m:
         #print "m.groups() = {}".format(m.groups())
         NX,NY,NZ,Angle = map(float,m.group(1).strip().split())
         zoom, TrX, TrY = map(float,m.group(2).strip().split())
         CX,CY,CZ = map(float,m.group(3).strip().split())
         print ":axis_norm = ({}, {}, {})  axis_angle = {}    zoom = {}  trxy = ({},{})   c = ({},{},{})".format(NX,NY,NZ,Angle, zoom, TrX, TrY, CX,CY,CZ)
         continue
      #m = re.match("load \/\*file\*\/\"(.*)\";", Line)
      m = re.match("load \"(.*)\";", Line)
      if m:
         FileNameXyz = m.group(1)
         if FileNameXyz.startswith('file:'):
            FileNameXyz = FileNameXyz[5:]
         continue
   assert(FileNameXyz is not None)
   print "   input xyz: '%s'" % (FileNameXyz)
   m = FMolecule(ReadXyzFile(FileNameXyz,GeometryScale))
   if Marked == list(range(1,1+len(m.Atoms))):
      Marked = []  # by default jmol selects all atoms. That is usually not what we want.
   AX = array([[ 0, 0, 0],
               [ 0, 0, 1],
               [ 0,-1, 0]])
   AY = array([[ 0, 0,-1],
               [ 0, 0, 0],
               [ 1, 0, 0]])
   AZ = array([[ 0, 1, 0],
               [-1, 0, 0],
               [ 0, 0, 0]])

   C = array([CX,CY,CZ])
   N = array([NX,NY,NZ]); N *= dot(N,N)**(-.5)
   #print "N = %s  Angle = %s  C = %s" % (N,Angle,C)
   R = expm(Angle/180. * pi * (AX*N[0] + AY*N[1] + AZ*N[2]))
   #print R
   Render( OutputFile, m, C+100*R[:,2], C, R[:,1] )


def ProcessArgs(argv):
   def ParseCmd(s,Cmd,nArgsExpected,Args):
      assert(nArgsExpected in [0,1])
      if nArgsExpected == 0:
         return Cmd == s
      else:
         if Cmd == s:
            iArg[0] += 1
            del Args[0:len(Args)]
            Args.append(argv[iArg[0]])
            return True
         elif Cmd.startswith('%s='%s):
            del Args[0:len(Args)]
            Args.append(Cmd[1+len(s):])
            return True
      return False

   iArg = [1]
   OutputFile = None
   DidSomething = False
   while iArg[0] < len(argv):
      Cmd = argv[iArg[0]]
      Args = []
      if ParseCmd('-o',Cmd,1,Args):
         OutputFile = Args[0]
      elif ParseCmd('--halos',Cmd,1,Args):
         global Halos
         global HaloBrightness
         if Args[0] in ['yes','no','true','false']:
            Halos = bool(Args[0])
         else:
            try:
               fHaloModUp = float(Args[0])
               fHaloModDown = -fHaloModUp/4
            except ValueError,e:
               ls = Args[0].split(':')
               assert(len(ls)==2) # up, down.
               fHaloModUp = float(ls[0])
               fHaloModDown = float(ls[1])
            HaloBrightness = [fHaloModUp, fHaloModDown]
            Halos = True
      elif ParseCmd('--mirror',Cmd,1,Args):
         global MirrorAxis
         MirrorAxis = Args[0]
         assert(re.match('x?y?z?',MirrorAxis) is not None)
      elif ParseCmd('--atom-scale',Cmd,1,Args):
         global AtomScale
         AtomScale = float(Args[0])
      elif ParseCmd('--bond-scale',Cmd,1,Args):
         global BondScale
         BondScale = float(Args[0])
      elif ParseCmd('--bond-thinning',Cmd,1,Args):
         global BondThinning
         BondThinning = float(Args[0])
      elif ParseCmd('--line-width',Cmd,1,Args):
         global BaseLineWidth
         BaseLineWidth = float(Args[0])
      elif ParseCmd('--bond-brightness',Cmd,1,Args):
         global BondBrightness
         BondBrightness = float(Args[0])
      elif ParseCmd('--elem-colors',Cmd,1,Args):
         # format:  --elem-colors=H:ffffff,C:604444,...
         ColorSets = Args[0].split(',')
         for ColorSet in ColorSets:
            ls = ColorSet.split(':')
            assert(len(ls) == 2)
            Elem,Color = tuple(ls)
            if Elem not in ElementNames:
               print "WARNING: Element '%s' not recognized (uppercase/lowercase okay?). Inserting it anyway in case I see it." % Elem
            AtomColors[Elem] = HC(Color)
      elif ParseCmd('--geom-scale',Cmd,1,Args):
         global GeometryScale
         if ( Args[0] == 'bohr' ):
            GeometryScale = ToAng
         elif ( Args[0] == 'ang' ):
            GeometryScale = 1.
         else:
            GeometryScale = float(Args[0])
            assert(GeometryScale != 0.)
      elif ParseCmd('--atom-labels',Cmd,1,Args) or ParseCmd('--labels',Cmd,1,Args):
         global MakeAtomLabels
         global MakeAtomLabelsAlsoForH
         global MakeAtomLabelsAlsoForC
         Arg = Args[0]
         if Arg == 'all':
            MakeAtomLabels = True
            MakeAtomLabelsAlsoForC = True
            MakeAtomLabelsAlsoForH = True
         elif Arg == 'yes':
            MakeAtomLabels = True
         elif Arg == 'no':
            MakeAtomLabels = False
         elif Arg == 'noHC':
            MakeAtomLabels = True
            MakeAtomLabelsAlsoForC = False
            MakeAtomLabelsAlsoForH = False
         elif Arg == 'noH':
            MakeAtomLabels = True
            MakeAtomLabelsAlsoForC = True
            MakeAtomLabelsAlsoForH = False
         else:
            raise Exception("Argument '%s' to --atom-labels not recognized. I know 'all','yes','no','noHC','noH'. " % Arg)
         if not MakeAtomLabels:
            MakeAtomLabelsAlsoForC = False
            MakeAtomLabelsAlsoForH = False
      elif ParseCmd('--bond-thr',Cmd,1,Args):
         global BondThreshold
         BondThreshold = float(Args[0])
      elif Cmd.startswith('-'):
         raise Exception("Option not recognized: '%s'" % Cmd)
      else:
         FileName = Cmd
         assert(path.splitext(FileName)[1] == '.jmol')
         RenderJmolState(FileName, OutputFile)
         OutputFile = None
         DidSomething = True
      iArg[0] += 1
   if not DidSomething:
      print "Usage: renmol <options> <jmol-state-file>"
      print "Getting a jmol state file:"
      print "   - Run jmol <xyz-file>"
      print "   - Uncheck 'Diplay/[x] Perspective Depth' in the menu"
      print "   - Orient your molecule"
      print "   - Use 'File/Export/Write State...' to save a state file."
      print "renmol should then make a picture for the molecule in the view."
      print
      print "Options:"
      print "   - Check ProcessArgs(argv) for possibilities"
      print "     (or hack up the script directly)"
      print "Notes:"
      print "   - Needs asymptote vector graphics scripting installed (command ``asy'')."
      print "   - This program is not bullet proof. It might need some tweeking"
      print "     to get it to do what you want."

if __name__ == "__main__":
   ProcessArgs(argv)

# kate: space-indent on; indent-width 3;
